/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "uiHandler.h"
#include "util/util.h"
#include <stdlib.h>

#if defined HAVE_STRING_H || defined _LIBC
# include <string.h>
#else
# include <strings.h>
#endif

struct func_list
{
	void (*init)(int*, char***);
	int (*is_commandline)();
	void (*print_to_console)(const char*, const char*);
	void (*draw_box)();
	void (*takes_over)();
	void (*msg)(const char*);
	void (*time)(const char*);
	void (*size)(long, long);
	void (*rbc)(int, int, int);
	void (*close)();
};

extern void CLIUI_init(int*, char***);
extern int CLIUI_is_commandline();
extern void CLIUI_print_to_console(const char*, const char*);
extern void CLIUI_takes_over();
extern void CLIUI_msg (const char*);
extern void CLIUI_time (const char*);
extern void CLIUI_size (long, long);
extern void CLIUI_rbc (int, int, int);
extern void CLIUI_close();
extern void CursesUI_init(int*, char***);
extern int CursesUI_is_commandline();
extern void CursesUI_print_to_console(const char*, const char*);
extern void CursesUI_draw_box();
extern void CursesUI_takes_over();
extern void CursesUI_msg (const char*);
extern void CursesUI_time (const char*);
extern void CursesUI_size (long, long);
extern void CursesUI_rbc (int, int, int);
extern void CursesUI_close();
extern void GtkUI_init(int*, char***);
extern int GtkUI_is_commandline();
extern void GtkUI_print_to_console(const char*, const char*);
extern void GtkUI_draw_box();
extern void GtkUI_takes_over();
extern void GtkUI_msg (const char*);
extern void GtkUI_time (const char*);
extern void GtkUI_size (long, long);
extern void GtkUI_rbc (int, int, int);
extern void GtkUI_close();

static struct func_list cli_funcs_ = 
{
	&CLIUI_init,
	&CLIUI_is_commandline,
	&CLIUI_print_to_console,
	0,
	&CLIUI_takes_over,
	&CLIUI_msg,
	&CLIUI_time,
	&CLIUI_size,
	&CLIUI_rbc,
	&CLIUI_close,
};

static struct func_list curses_funcs_ =
#ifndef UI_CURSES
{
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
};
#else
{
	&CursesUI_init,
	&CursesUI_is_commandline,
	&CursesUI_print_to_console,
	&CursesUI_draw_box,
	&CursesUI_takes_over,
	&CursesUI_msg,
	&CursesUI_time,
	&CursesUI_size,
	&CursesUI_rbc,
	&CursesUI_close,
};
#endif

static struct func_list gui_funcs_ =
#ifndef UI_GTK
{
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
};
#else
{
	&GtkUI_init,
	&GtkUI_is_commandline,
	&GtkUI_print_to_console,
	&GtkUI_draw_box,
	&GtkUI_takes_over,
	&GtkUI_msg,
	&GtkUI_time,
	&GtkUI_size,
	&GtkUI_rbc,
	&GtkUI_close,
};
#endif

static struct func_list *current_ui_;

static void control_c () { Exception_fatal (_("Control-C hit.")); }

void UIHandler_init()
{
  sigint_handler (control_c);
  cli_funcs_.init (0, 0);
  current_ui_ = &cli_funcs_;
}

void UIHandler_change_to_gui_if_possible()
{
  void (*func)();
  /* we have to assume that Config_init() was called now */

  if (Config_iget ("INVISIBLE"))
  	return;
  
  if (Config_iget ("NO_GUI") || !gui_funcs_.init) {
#ifndef UI_CURSES
  	return;
#endif
  	cli_funcs_.close();
  	current_ui_ = &curses_funcs_;
  	curses_funcs_.init (0, 0);
  }
  else {
  	cli_funcs_.close();
  	current_ui_ = &gui_funcs_;
  }
  
  func = current_ui_->draw_box;
  assert (func);
  func();
}

void UIHandler_parse_gui_options (int* c, char*** v)
{
  if (gui_funcs_.init) 
  	gui_funcs_.init (c, v);
}

int UIHandler_is_commandline()
{
  int (*func)() = current_ui_->is_commandline;
  assert (func);
  return func();
}

void UIHandler_print_to_console (const char* s1, const char* s2)
{
  void (*func)() = current_ui_->print_to_console;
  assert (func);
  func (s1, s2);
}

void UIHandler_takes_over()
{
  void (*func)() = current_ui_->takes_over;
  assert (func);
  func();
}

void UIHandler_msg (const char* m)
{
  void (*func)() = current_ui_->msg;
  assert (func);
  func (m);
}

void UIHandler_time (const char* m)
{
  void (*func)() = current_ui_->time;
  assert (func);
  func (m);
}

void UIHandler_size (long l1, long l2)
{
  void (*func)() = current_ui_->size;
  assert (func);
  func (l1, l2);
}

void UIHandler_rbc (int r, int b, int c)
{
  void (*func)() = current_ui_->rbc;
  assert (func);
  func (r, b, c);
}

void UIHandler_close()
{
  void (*func)() = current_ui_->close;
  assert (func);
  func();
}

